# Sendmail.pm -- LocalMTA::Sendmail module file. See LocalMTA/Sendmail.pod.

# This file is part of LocalMTA-Sendmail, and is released under the GPLv2. Copyright 2013-2014 by Troye Johnson.

package LocalMTA::Sendmail;

use warnings;
use strict;

use Carp qw(croak);

use Exporter;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw(sendmail localmta_sendmail email_valid localmta_valid);

our $VERSION = '0.04';

## used to verify emails
sub email_valid {
    return 1 if ((defined $_[0]) && ($_[0] =~ /^([a-zA-Z0-9_\.-]+)@([a-zA-Z0-9_\.-]+)$/)); # mail to Internet
    return 1 if ((defined $_[0]) && ($_[0] =~ /^([a-zA-Z0-9_\.-]+)$/)); # local
    return 0;
} # email_valid()

# ** localmta_valid: this is just another name for email_valid().
sub localmta_valid {
    my ($email_addr) = @_;

    # remove all characters safe for shells
    $email_addr =~ s/(@|-|_|\.)//g;

    return undef if ($email_addr =~ /\W/); # if bad chars remain, signal error
    return 1; # email_addr is good to go
} # localmta_valid


=item

    sendmail({mta_path => '/usr/sbin/exim4', local_mail_flg => '-bm'}, $to, $subject, $msg);

Returns: true or false

Parameters:
 - HASH, 'mta_path' must be present in parameter, it is the full path the MTA executable on your system. Do an 'ls -l /usr/sbin/sendmail' to see what it really is. 'local_mail_flag' must also be defined in the HASH. It is the flag used to tell the MTA to accept an email message over STDIN. Not all MTAs need that flag. Courier MTA does not need it, but Exim does (exim will refuse to send messages without '-bm' on command line).
 - $to, is the recipient. It the future, $to can may be an ARRAY reference, so that way you can send mail to multiple recipients at a time.
 - $subject, is the subject of the email message (string)
 - $msg, is the body of the email message to send via you local MTA

Note: you will need to configure the 'trusted users' list, so that the $full_from, and $from options will be honored. Else, the MTA will put whatever it thinks is reasonable. Only Exim4 needs this.

This module can send mail to multiple people at the same time, up to 128 recipients can be passed in per call to localmta_sendmail. The specific number is the command line limit on your system. For most GNU/Linux distributions this is about 3278 characters.

This module can send 6000 e-mails per second (or 360,000 per hour).

=cut

sub sendmail {
    croak 'sendmail: must have six parameters' unless ($#_ == 5);
    croak 'sendmail: first param must be HASH reference' unless (ref $_[0]);

# todo: $to can be an array reference. in such a case, the $msg is sent to all recipients in @$to.
    my ($opts, $to, $full_from, $from, $subject, $msg) = (@_);

    # This prevents some "concatenation of undefined value" errors
    $opts->{local_mail_flag} = "" 
	unless defined $opts->{local_mail_flag};

    # E-mail address must be valid, and not contain dangerous shell characters
    if (ref $to) {
	# Check that each e-mail address is valid and safe for sh(1)
	for(@$to) {
	    return 0 unless (localmta_valid($_));
	}
    }
    else {
	# Check just one address
	return 0 unless (localmta_valid($to));
    }

    # MTA program must exist
    return 0 unless (-e $opts->{'mta_path'}); 

    # Build e-mail message headers, and append $msg
    my $finalmsg = "";
    # $finalmsg = 'To: ' . $to . "\n";
    $finalmsg .= 'Cc: ' . $opts->{'cc'} . "\n"
	if defined $opts->{'cc'};  # add xtra rcpnt if present -- xfmod
    $finalmsg .= 'Subject: ' . $subject . "\n\n";
    $finalmsg .= $msg . "\n";  

    local $SIG{'PIPE'} = 'IGNORE';
    local $SIG{'CHLD'} = 'IGNORE';
    
    # Send e-mail(s)
    if (ref $to) {
 	# sendmail to all recipients addresses in $to (ARRAY reference)
	open( my $pipe_email, 
	      "| $opts->{'mta_path'} $opts->{'local_mail_flag'} -f $from @$to")
	    or return;

	print $pipe_email $finalmsg or return;	
    }
    else {
	# sendmail to just one recipient
	open( my $pipe_email, 
	      "| $opts->{'mta_path'} $opts->{'local_mail_flag'} -f $from $to")
	    or return;

	print $pipe_email $finalmsg or return;	
	close $pipe_email;
    }
    my $return_value = $?;

    # Exim does not like to accept local messages. (See -bm and -oe[x] options
    # in man page or Exim4 manual). Exim returns -1 (failure) even though
    # message goes through. We account for that here.
    my $okay_retval = -1;
    if ( $return_value == $okay_retval || $return_value == 0)  {
	return 1; # successful return
    } 

    return 0; # failure
} # sendmail


# localmta_sendmail
#
# Just another name for LocalMTA::Sendmail::sendmail(). See docs for sendmail(). This is same function, but with a different name.

sub localmta_sendmail {
    return sendmail(@_);
} # localmta_sendmail


1; # ok load
